'use strict';
const BaseService = require('./base-service');
/**
 * 数据库Service基类
 */
class DbService extends BaseService {
  // 获取helper里的数据库对象
  get db() {
    return this.helper.db;
  }
  // 为直接使用mysql模块的escape
  get mql() {
    return require('mysql');
  }
  /**
   * 直接用db1执行sql
   * @param {string} _sql sql语句
   * @return {Promise} Promise
   */
  async doQuery(_sql) {
    return await this.db.doQuery(_sql, this);
  }
   /**
   * 直接用db1执行sql 如果查不到就返回null，查到直接返回对象
   * @param {string} _sql sql语句
   * @return {Promise} 数据对象
   */
  async doQueryObj(_sql){
    let result = await this.db.doQuery(_sql, this);
    if(result.length == 0) return null
    return result[0]
  }
  /**
   * 分页查询
   * @param {object} info 
   * @param {string} info.sql sql语句
   * @param {number} info.page_num  页码默认1
   * @param {number} info.page_size 一页多少条默认10 
   */
  async doPage(info){
    this.ctx.validate({
      _sql : {type:'string'}
    },info)
     let list = []
     let count = 0 
     let page_num = info.page_num || 1
     let page_size = info.page_size || 10
     let that = this
     await Promise.all([
         async function(){
             list = await that.doQuery(`${info._sql} limit ${ that.to((page_num-1)*page_size ) } , ${that.to(page_size)}`)
         }(),
         async function(){
             let c = await that.doQuery(
                 `select count(*) as c from ( ${info._sql}) c`
             )
             if(c.length>0){
                 count =c[0].c 
             }
         }()
     ])
     return {
       list,
       count
     }
}
  /**
   * 直接用db1执行sql 抛出错误
   * @param {string} _sql sql语句
   * @return {Promise} Promise
   */
  async doQueryError(_sql) {
    return await this.db.doQueryError(_sql, this);
  }
  
  /**
   * 执行插入操作并自动插入创建时间，最后更新时间
   * 需要表中有create_at(varchar(13))、update_at(varchar(13))字段
   * @param {string} _sql 插入sql语句
   */
  async doInsert(_sql){
    const that = this
    return await this.tranError(async conn=>{
       const result =  await that.doQueryConnError(conn,_sql) 
        if(result.affectedRows>0){
            const time = Date.now()
            const tableName = _sql.split('into')[1].split('(')[0]
            await that.doQueryConnError(conn,
                `update ${tableName} set create_at=${that.to(time)},update_at=${that.to(time)} where id = ${that.to(result.insertId)}`)
        }
        return result
    })
 }
 /**
  * 执行查询并自动增加create_time,update_time字段格式为YYYY-MMM-DD HH:mm:ss
  * 需要表中有create_at(varchar(13))、update_at(varchar(13))字段
  * @param {string} _sql 查询sql语句
  */
 async doList(_sql){
  return await this.doQuery(
      `select _dalitek.*,from_unixtime(CONVERT(create_at/1000, UNSIGNED INTEGER),'%Y-%m-%d %H:%i:%S') as create_time,from_unixtime(CONVERT(update_at/1000, UNSIGNED INTEGER),'%Y-%m-%d %H:%i:%S') as update_time from ( ${_sql} ) _dalitek`
  )
 }
 /**
  * 执行更新操作并自动将update_at字段更新为当前时间
  * 需要表中有update_at(varchar(13))字段
  * @param {string} _sql 更新sql语句
  */
 async doUpdate(_sql){
  let _sqlStrArr = _sql.split('where')
  if(_sqlStrArr.length==0){
      _sqlStrArr = _sql.split('WHERE')
  }
  if(_sqlStrArr.length == 0){
      return {
          affectedRows : 0
      }
  }
  _sql =  `${_sqlStrArr[0]} ,update_at=${this.to(Date.now())} where ${_sqlStrArr[1]}`
  return await this.doQueryError(_sql)
}
/**
 * 执行更新is_delete=1操作并自动将update_at字段更新为当前时间
 * @param {string} _sql 更新is_delete sql语句
 */
async doDel(_sql){
  let _sqlStrArr = _sql.split('where')
  if(_sqlStrArr.length==0){
      _sqlStrArr = _sql.split('WHERE')
  }
  if(_sqlStrArr.length == 0){
      return {
          affectedRows : 0
      }
  }
  _sql =  `${_sqlStrArr[0]} ,update_at=${this.to(Date.now())} where ${_sqlStrArr[1]}`
  return await this.doQueryError(_sql)
}


  /**
   * 在tran执行事务的时候使用可以传递统一的conn数据库连接
   * @param {object} conn 数据库连接
   * @param {string} _sql sql语句
   *  @return {Promise} Promise
   */
  async doQueryConn(conn, _sql) {
    return await this.db.doQueryConn(_sql, conn, this);
  }
   /**
   *  在tran执行事务的时候使用可以传递统一的conn数据库连接 如果查不到就返回null，查到直接返回对象
   * @param {object} conn 数据库连接
   * @param {string} _sql sql语句
   * @return {Promise} 数据对象
   */
  async doQueryConnObj(conn, _sql){
    let result = await this.db.doQueryConn(_sql, conn,this);
    if(result.length == 0) return null
    return result[0]
  }

  /**
   * 在tran执行事务的时候使用可以传递统一的conn数据库连接 抛出错误
   * @param {object} conn 数据库连接
   * @param {string} _sql sql语句
   *  @return {Promise} Promise
   */
  async doQueryConnError(conn, _sql) {
    return await this.db.doQueryConnError(_sql, conn, this);
  }
  /**
   * 执行事务
   * @param {asyncFunction} asyncFunction 事务执行 async回调方法
   * @return {Promise} Promise
   */
  async tran(asyncFunction) {
    return await this.db.tran(this, asyncFunction);
  }
  /**
   * 执行事务 抛出Error
   * @param {asyncFunction} asyncFunction 事务执行 async回调方法
   * @return {Promise} Promise
   */
  async tranError(asyncFunction) {
    return await this.db.tranError(this, asyncFunction);
  }
  /**
   * 执行多数据源sql
   * @param {string} dbId 数据源id
   * @param {string} _sql sql语句
   *  @return {Promise} Promise
   */
  async doMQuery(dbId, _sql) {
    const conn = this.app.mysql.get(dbId);
    return await this.db.doQueryConn(_sql, conn, this);
  }
  /**
   * 执行多数据源事务
   * @param {string} dbId 数据源id
   * @param {string} asyncFunction 事务执行 async回调方法
   * @return {Promise} Promise
   */
  async tranM(dbId, asyncFunction) {
    const conn = this.app.mysql.get(dbId);
    return this.db.tranConn(this, conn, asyncFunction);
  }
  /**
   * 执行多数据源事务 抛出错误
   * @param {string} dbId 数据源id
   * @param {string} asyncFunction 事务执行 async回调方法
   * @return {Promise} Promise
   */
  async tranMError(dbId, asyncFunction) {
    const conn = this.app.mysql.get(dbId);
    return this.db.tranConnError(this, conn, asyncFunction);
  }
  /**
   * 防sql注入处理
   * @param {object} params sql中需要处理的字段
   * @return {object} object
   */
  to(params) {
    return this.mql.escape(params);
  }
}
module.exports = DbService;
